/*
    mdw - md5sums a web resource
    Copyright (C) 2001  colmmacc@redbrick.dcu.ie

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <netdb.h>
#include "mdw_http.h"
#include "mdw_arg.h"
#include "mdw_aux.h"


/* show the url in the output ? */
BOOL		mdw_show_url = TRUE;

/* show the sum in the output ? */
BOOL		mdw_show_sum = TRUE;

/* show the "MATCHED" | "CHANGED" text ? */
BOOL		mdw_show_matched = TRUE;

/* use a proxy ? */
BOOL		mdw_use_proxy = UNSET;

/* our user agent name */
char	*	mdw_user_agent="mdw";

/* the proxy (if any) */
uri_t	*	mdw_uri_proxy;	

/* the linked list of uri's to check */
uri_t	*	mdw_uri_list = NULL;

/* parse the arguments list */
void mdw_parse_argv(int argc, char **argv)
{
	int 		option;
	uri_t   * 	next,
		*	ptr;


	opterr = 0;
	/* parse all the options */
	while ((option = getopt(argc, argv, "p:a:scueqQm")) != EOF) {
		switch(option) {
			/* use a specified proxy */
			case 'p':	if(mdw_use_proxy!=UNSET) {
						fprintf(stderr, "conflicting options\n\n");
						usage();
					} else {
						mdw_use_proxy=FALSE;
						mdw_uri_proxy=uri_parse(optarg);
						if(mdw_uri_proxy->uri_error==TRUE) {
							fprintf(stderr,"invalid proxy: %s\n\n", optarg);
						} else {
							mdw_use_proxy=TRUE;
						}
					}	
					break;

			/* use this user-agent */
			case 'a':	mdw_user_agent=strdup(optarg);
					break;

			/* suppress use of proxy */
			case 's':	if(mdw_use_proxy==TRUE) {
						fprintf(stderr, "conflicting options\n\n");
						usage();
					} else {
						mdw_use_proxy=FALSE;
					}
					break;

			/* hide the checksum */
			case 'c':	mdw_show_sum=FALSE;
					break;

			/* hide the uri */
			case 'u':	mdw_show_url=FALSE;
					break;
			
			/* hide the match string */
			case 'm':	mdw_show_matched=FALSE;
					break;

			/* hide all output */
			case 'Q':	if(!(freopen("/dev/null","w", stderr))) {
						fprintf(stderr,"Error Opening /dev/null\n");
						exit(FALSE);
					}
					/* fall thru */

			/* hide  standard output */
			case 'q':	if(!(freopen("/dev/null","w", stdout))) {
						fprintf(stderr,"Error Opening /dev/null\n");
						exit(FALSE);
					}
					break;


			/* hide errors */
			case 'e':	if(!(freopen("/dev/null","w", stderr))) {
						fprintf(stderr,"Error Opening /dev/null\n");
						exit(FALSE);
					}
					break;
			case 'h':
			case 'H':
			case '?':
			case 'V':
			default:	usage();
					break;
		}
	}


	/* use the env to set the proxy if it mkes sense */
	if(mdw_use_proxy==UNSET) {

		mdw_use_proxy=FALSE;

		/* less common but used */
		if(getenv("http_proxy")) {
			if((mdw_uri_proxy=uri_parse(getenv("http_proxy")))==NULL) {
				fprintf(stderr,"environment variable http_proxy is invalid, ignoring\n\n");
			} else {
				mdw_use_proxy=TRUE;
			}
		}

		/* standard method */
		if(getenv("HTTP_PROXY")) {
			if((mdw_uri_proxy=uri_parse(getenv("HTTP_PROXY")))==NULL) {
				fprintf(stderr,"environment variable HTTP_PROXY is invalid, ignoring\n\n");
			} else {
				mdw_use_proxy=TRUE;
			}
		}

	}


	/* check for url's */
	if (optind == argc) {
		usage();
	}

	/* add the uri */	

	do {

		/* skip to next uri position */
		ptr=mdw_uri_list;
		while(ptr != NULL && ptr->next!=NULL) {
			ptr=ptr->next;
		}

		/* parse the string */
		next=uri_parse(argv[optind]); 

		/* check if there are at least 3 args left and
		   for the presence of the comparison operator */
		if( (argc - optind ) > 2 && 
		    (argv[optind+1][0]=='=') ){
				if(check_md5(argv[optind+2])==FALSE) {
					fprintf(stderr,"invalid checksum: %s\n", argv[optind+2]);		
					next->uri_error=TRUE;
				} 

				next->uri_match=argv[optind+2];

				if(mdw_uri_list==NULL) {
					mdw_uri_list=next;
				} else {
					ptr->next=next;	
				}
				
				optind+=2;
		} else if(mdw_uri_list==NULL) {
			mdw_uri_list=next;
		} else {
			 ptr->next=next;
		}

		optind++;

		next->next=NULL;

	} while(optind < argc);



}

/* the usage statement */
void usage(void) 
{

	fprintf(stderr,
		"usage: mdw [options] url [= checksum] ...\n\n"
		"\t-p proxy:port : use specified proxy\n"
		"\t-s            : dont use a proxy\n"
		"\t-a agent-name : pretend to be specified user-agent\n"
		"\t-c            : dont display checksum\n"
		"\t-u            : dont display url\n"
		"\t-m            : dont display MATCHED|CHANGED|ERROR|OK string\n"
		"\t-e            : hide all errors\n"
		"\t-q            : hide standard output\n"
		"\t-Q            : hide all output\n\n"
		"mdw by colmmacc@redbrick.dcu.ie, (C) 2001\n");

	
	exit(FALSE);
}

