/*
    mdw - md5sums a web resource
    Copyright (C) 2001  colmmacc@redbrick.dcu.ie

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include <string.h>
#include <strings.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <netdb.h>

#include "mdw_http.h"
#include "mdw_aux.h"
#include "mdw_arg.h"
#include "mdw_sock.h"



/* turn a uri string into a uri_t structure */
uri_t * uri_parse(char * string)
{
	uri_t	*	ptr;
	char	*	spare;

	/* can't fail */
	ptr=alloc_uri();	

	/* set some stuff */
	ptr->uri_string=strdup(string);
	ptr->uri_match=NULL;
	ptr->uri_error=FALSE;
	ptr->next=NULL;
	ptr->uri_port=80;
	ptr->uri_request="/";


	if(mdw_use_proxy==TRUE) {
		/* set a full canonical request */
		if(!strncmp("http://", string, 7)) {
			ptr->uri_request=strdup(string);
		} else {
			if((ptr->uri_request=malloc(8 + strlen(string)))==NULL) {
				fprintf(stderr,"out of memory\n");
				exit(FALSE);
			} else {
				sprintf(ptr->uri_request,"http://%s", string);
			}
		}
	}

	/* parse out the hostname */

	/* skip the http:// */
	if(!strncmp("http://", string, 7)) {
		string+=7;
	}
	

	/* get the host stuff */
	/* get the request uri */
	if((spare=index(string, '/'))) {

		/* copy the request */
		if(mdw_use_proxy==FALSE) {
			ptr->uri_request=strdup(spare);
		}
		/* terminate the string at the first slash */
		*spare='\0';
	}

	/* string is now either hostname, or hostname:port */	
	if((spare=index(string, ':'))) {
			
		/* get the port number out */
		if(*(spare+1)) {
			ptr->uri_port=atoi(spare + 1);
		} else {
			fprintf(stderr, "no port number supplied\n");
		}
		/* terminate the hostname */
		*spare='\0';
	}
	
	/* now get the host */		
	if(mdw_use_proxy==FALSE) {
		if((ptr->uri_host=gethostbyname(string))==NULL) {
			fprintf(stderr,"No such host: %s\n", string);
			ptr->uri_error=TRUE;
		}
	}
	
	/* now copy the hostname */
	ptr->uri_hostname=strdup(string);	

	return ptr;

}

/* handle the http connection  (including redirects)
   and exit at start of content */
int http_request(uri_t * request)
{
	int	fd;
	char	request_str[1024];	
	char	*line;
	char	new_uri[1024];
	int	code;	


	/* connect() the socket */
	if((fd=connect_uri(request))>=0) {
	
		/* make the request */	
		snprintf(request_str,1024,
			"GET %s HTTP/1.0\r\n"
			"Host: %s\r\n"
			"User-Agent: %s\r\n\r\n", 
			request->uri_request, 
			request->uri_hostname,
			mdw_user_agent);

		/* send the request */
		writen(fd, request_str, strlen(request_str));

		/* read the reply */
		if((line=readline(fd))!=NULL&&
		   (sscanf(line, "HTTP/%*d.%*d %d %*[.*]", &code)==1)) {
			/* is a httpd, do stuff with the reply */
			switch(code) {

				/* we have the document */
				/* skip all the headers and exit */
				case 200:	while((line=readline(fd))) {
							if(line[0]==' ' || 
							   line[0]=='\n' ||
							   line[0]=='\r' ||
							   line[0]=='\t' ) {
								return fd;
							}

						} 

						fprintf(stderr,"%s: Document has no body\n", request->uri_string);
						request->uri_error=TRUE;
						return -1;
						break;	

				/* Document is somewhere else */
				/* get the new location and go there */
				case 301:
				case 302:	while((line=readline(fd))) {
							if(sscanf(line, "Location: %1024s", new_uri)) {
								request=uri_parse(new_uri);
								return http_request(request);	
							}
						}

						fprintf(stderr,"%s : invalid http 302 headers\n", request->uri_string);
						request->uri_error=TRUE;
						return -1;
						break;
				/* an error */
				default:	
						fprintf(stderr,"%s : http error (%d)\n", request->uri_string ,code);
						request->uri_error=TRUE;
						return -1;
						break;	
			}

		} else {
			/* prob not httpd */
			fprintf(stderr, "http error\n");
			request->uri_error=TRUE;
			return -1;
		}
	}

	


	return fd;
}

